#!/usr/bin/env python
# -*- encoding: utf-8 -*-

# Copyright 2011-2018, Nigel Small
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__all__ = ["__author__", "__email__", "__license__", "__package__", "__version__",
           ]

__author__ = "FDSEKG"
__email__ = "lmwtclmwtc@outlook.com"
__license__ = "Apache License, Version 2.0"
__package__ = "sekg"
__version__ = "0.4.31"
